package com.hyp.web.controller.reward;

import com.hyp.common.annotation.Log;
import com.hyp.common.config.HypConfig;
import com.hyp.common.constant.Constants;
import com.hyp.common.core.controller.BaseController;
import com.hyp.common.core.domain.AjaxResult;
import com.hyp.common.enums.BusinessType;
import com.hyp.common.exception.ServiceException;
import com.hyp.common.utils.HypFileUtil;
import com.hyp.common.utils.file.FileUploadUtils;
import com.hyp.common.utils.file.FileUtils;
import com.hyp.framework.config.ServerConfig;
import com.hyp.system.domain.RewFileRelation;
import com.hyp.web.controller.manager.FileRelationManager;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.List;
import java.util.Objects;


/**
 * packageName com.hyp.web.controller.reward
 *
 * @author wangxy
 * @version JDK 8
 * @className FileRelationController
 * @date 2024/4/12
 * @description 文件
 */
@Api("文件")
@RestController
@RequestMapping("/reward/file")
@Slf4j
public class FileRelationController extends BaseController {


    @Resource
    private FileRelationManager fileRelationManager;

    @Resource
    private ServerConfig serverConfig;

    @ApiOperation("文件上传")
    @PostMapping("/upload")
    public AjaxResult uploadFile(MultipartFile file,
                                 @RequestParam String applyId,
                                 @RequestParam(required = false) String filename,
                                 @RequestParam String fileCode) {
        try {
            if(file.isEmpty()){
                throw new ServiceException("上传文件不能为空");
            }
            // 上传文件路径
            String filePath = HypConfig.getUploadPath();
            // 上传并返回新文件名称
            String fileNames = FileUploadUtils.upload(filePath, file);
            String url = serverConfig.getUrl() + fileNames;
            if(StringUtils.isBlank(filename)){
                filename = fileNames;
            }
            String originalFilename = FileUtils.getNameNotSuffix(file.getOriginalFilename()) ;
            return  AjaxResult.success(fileRelationManager.save(applyId,filename,fileCode,url,originalFilename));
        } catch (Exception e) {
            return AjaxResult.error(e.getMessage());
        }
    }

    /**
     * 通用上传请求（多个）
     */
    @ApiOperation("多文件上传")
    @PostMapping("/uploads")
    public AjaxResult uploadFiles(List<MultipartFile> files,
                                  @RequestParam String applyId,
                                  @RequestParam(required = false) String filename,
                                  @RequestParam String fileCode){
        try {
            return  AjaxResult.success(fileRelationManager.saveBatch(files,applyId,filename,fileCode));
        } catch (Exception e) {
            return AjaxResult.error(e.getMessage());
        }
    }

    @ApiOperation("材料预览")
    @GetMapping("fileView/{fileId}")
    public void fileView(@PathVariable String fileId, HttpServletResponse response) throws IOException {
        String filePath = fileRelationManager.getFilePath(fileId);
        String fileName = fileRelationManager.getRealName(fileId);
        String encodedFileName = new String(fileName.getBytes(StandardCharsets.UTF_8),StandardCharsets.ISO_8859_1);
        InputStream in = null;
        OutputStream out = null;
        try {
            response.setHeader("content-disposition", "attachment;filename=" + encodedFileName + ".pdf");
            // 设置文件ContentType类型，这样设置，会自动判断下载文件类型
            response.setContentType("application/octet-stream;charset=utf-8");
            File file = new File(filePath);
            in = Files.newInputStream(file.toPath());
            // 从response对象获取输出流
            out = response.getOutputStream();
            // 将输入流中的数据读取出来，写到输出流中
            for (int b = -1; (b = in.read()) != -1; ) {
                out.write(b);
            }
        } catch (Exception e) {
            log.error("获取文件流失败：", e.getMessage(), e);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                    log.error(e.getMessage(), e);
                } finally {
                    if (out != null) {
                        out.close();
                    }
                }
            }
        }
    }

    @ApiOperation("材料删除")
    @Log(title = "材料删除", businessType = BusinessType.DELETE)
    @GetMapping("/delFile/{fileId}")
    public AjaxResult delFile(@PathVariable String fileId){
        return  toAjax(fileRelationManager.delFile(fileId));
    }


    @ApiOperation("材料审核通过id")
    @GetMapping("/filePass")
    public AjaxResult pass(@RequestParam String fileId,
                               @RequestParam Integer fileState){
        return  toAjax(fileRelationManager.pass(fileId,fileState));
    }


    @ApiOperation("材料校验")
    @GetMapping("/checkFile/{applyId}")
    public AjaxResult checkFile(@PathVariable String applyId){
        return  fileRelationManager.checkFile(applyId);
    }

    @ApiOperation("文件查询")
    @GetMapping("/getFileInfo")
    public AjaxResult getFileInfo(@RequestParam String applyId,
                                   @RequestParam(required = false) String fileCode) {
        return success(fileRelationManager.getFileInfo(applyId,fileCode));
    }

    @ApiOperation("文件下载")
    @GetMapping("/download")
    public void fileDownload(String fileId, HttpServletResponse response, HttpServletRequest request) {
        try {
             RewFileRelation rewFileRelation = fileRelationManager.getRewFileRelation(fileId);
             if(Objects.isNull(rewFileRelation)){
                 throw new ServiceException("下载文件不存在");
             }
             String fileName = rewFileRelation.getFilePath();
            if (!FileUtils.checkAllowDownload(fileName)) {
                throw new ServiceException(com.hyp.common.utils.StringUtils.format("资源文件({})非法，不允许下载。 ", fileName));
            }
            // 本地资源路径
            String localPath = HypConfig.getProfile();
            // 数据库资源地址
            String downloadPath = localPath + StringUtils.substringAfter(fileName, Constants.RESOURCE_PREFIX);
            // 下载名称
            String downloadName = StringUtils.substringAfterLast(downloadPath, "/");
            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
            FileUtils.setAttachmentResponseHeader(response, downloadName);
            FileUtils.writeBytes(downloadPath, response.getOutputStream());
        } catch (Exception e) {
            log.error("下载文件失败", e);
        }
    }





}
